import 'package:appflowy/workspace/presentation/widgets/pop_up_action.dart';
import 'package:flutter/material.dart';
import 'package:appflowy/generated/locale_keys.g.dart';
import 'package:easy_localization/easy_localization.dart';

enum SmartEditAction {
  summarize,
  fixSpelling;

  String get toInstruction {
    switch (this) {
      case SmartEditAction.summarize:
        return 'Tl;dr';
      case SmartEditAction.fixSpelling:
        return 'Correct this to standard English:';
    }
  }

  String prompt(String input) {
    switch (this) {
      case SmartEditAction.summarize:
        return '$input\n\nTl;dr';
      case SmartEditAction.fixSpelling:
        return 'Correct this to standard English:\n\n$input';
    }
  }

  static SmartEditAction from(int index) {
    switch (index) {
      case 0:
        return SmartEditAction.summarize;
      case 1:
        return SmartEditAction.fixSpelling;
    }
    return SmartEditAction.fixSpelling;
  }

  String get name {
    switch (this) {
      case SmartEditAction.summarize:
        return LocaleKeys.document_plugins_smartEditSummarize.tr();
      case SmartEditAction.fixSpelling:
        return LocaleKeys.document_plugins_smartEditFixSpelling.tr();
    }
  }
}

class SmartEditActionWrapper extends ActionCell {
  final SmartEditAction inner;

  SmartEditActionWrapper(this.inner);

  Widget? icon(Color iconColor) => null;

  @override
  String get name {
    return inner.name;
  }
}
